@@ -32,7 +32,6 @@ __all__ = [
     'parse_config_file',
     'initialise_flask',
     'initialise_prometheus',
-    'initialise_prometheus_register',
     'generate_locale_selector',
     'CredentialManager',
 ]
@@ -184,6 +183,23 @@ def initialise_flask(name):
     RequestID(app)
     return app
 
+def pass_through(undecorated):
+    def decorator(*args, **kwargs):
+        return undecorated(*args, **kwargs)
+    decorator.__name__ = undecorated.__name__
+    return decorator
+
+class FakeMetrics:
+    def do_not_track(self):
+        return pass_through
+    def counter(self, *args, **kwargs):
+        return pass_through
+    def histogram(self, *args, **kwargs):
+        return pass_through
+    def gauge(self, *args, **kwargs):
+        return pass_through
+    def summary(self, *args, **kwargs):
+        return pass_through
 
 def initialise_prometheus(app, log=None):
     # Prometheus
@@ -192,28 +208,11 @@ def initialise_prometheus(app, log=None):
         if log:
             log.info("Prometheus metrics enabled")
         return metrics
-    return None
-
-
-def initialise_prometheus_register(metrics):
-    # Register routes with Prometheus - call after all routes set up.
-    if os.environ.get("prometheus_multiproc_dir", False):
-        metrics.register_default(
-            metrics.summary(
-                'flask_ows_request_method_and_layer', 'Request summary by protocol, version, command and layer',
-                labels={
-                    'query_request': lambda: request.args.get('request'),
-                    'query_service': lambda: request.args.get('service'),
-                    'query_version': lambda: request.args.get('version'),
-                    'query_layer': lambda: (request.args.get('layers')          # WMS
-                                            or request.args.get('layer')        # WMTS
-                                            or request.args.get('coverage')     # WCS 1.x
-                                            or request.args.get('coverageid')   # WCS 2.x
-                                            ),
-                }
-            )
-        )
+    return FakeMetrics()
 
+def request_extractor():
+    qreq = request.args.get('request')
+    return qreq
 
 def generate_locale_selector(locales):
     def selector_template():
