@@ -11,12 +11,13 @@ from typing import (Any, Iterable, List, Mapping, MutableMapping, Optional,
 import datacube.model
 import numpy as np
 import xarray as xr
+from flask_babel import get_locale
 from PIL import Image
 
 import datacube_ows.band_utils
-from datacube_ows.config_utils import (CFG_DICT, AbstractMaskRule, FlagBand,
-                                       FlagProductBands, OWSConfigEntry,
-                                       OWSEntryNotFound,
+from datacube_ows.config_utils import (CFG_DICT, RAW_CFG, AbstractMaskRule,
+                                       FlagBand, FlagProductBands,
+                                       OWSConfigEntry, OWSEntryNotFound,
                                        OWSExtensibleConfigEntry,
                                        OWSFlagBandStandalone,
                                        OWSIndexedConfigEntry,
@@ -28,6 +29,69 @@ from datacube_ows.ogc_utils import ConfigException, FunctionWrapper
 _LOG: logging.Logger = logging.getLogger(__name__)
 
 
+class LegendBase(OWSConfigEntry):
+    """
+    Legend base class.
+    """
+    def __init__(self, style_or_mdh: Union["StyleDefBase", "StyleDefBase.Legend"], cfg: CFG_DICT) -> None:
+        super().__init__(cfg)
+        raw_cfg = cast(CFG_DICT, self._raw_cfg)
+        self.style_or_mdh = style_or_mdh
+        if isinstance(self.style_or_mdh, StyleDefBase):
+            self.style: StyleDefBase = self.style_or_mdh
+        else:
+            self.style = self.style_or_mdh.style
+        self.show_legend = cast(bool, raw_cfg.get("show_legend", self.style_or_mdh.auto_legend))
+        self.legend_urls: MutableMapping[str, str] = self.parse_urls(raw_cfg.get("url"))
+
+        self.width: float = 0.0
+        self.height: float = 0.0
+        self.mpl_rcparams: MutableMapping[str, str] = {}
+        if self.show_legend and not self.legend_urls and self.style.auto_legend:
+            self.parse_common_auto_elements(raw_cfg)
+
+    def parse_urls(self, cfg: RAW_CFG) -> MutableMapping[str, str]:
+        if not cfg:
+            return {}
+        def_loc = self.global_config().default_locale
+        if isinstance(cfg, str):
+            cfg = {
+                def_loc: cfg
+            }
+        if def_loc not in cfg:
+            raise ConfigException(
+                f"No legend url for {self.get_obj_label()} supplied for default language {def_loc}"
+            )
+        urls = {}
+        for locale in self.global_config().locales:
+            if locale in cfg:
+                urls[locale] = cfg[locale]
+            else:
+                urls[locale] = cfg[def_loc]
+        return urls
+
+    def parse_common_auto_elements(self, cfg: CFG_DICT):
+        self.width = cast(float, cfg.get("width", 4.0))
+        self.height = cast(float, cfg.get("height", 1.25))
+        self.mpl_rcparams = cast(MutableMapping[str, str], cfg.get("rcParams", {}))
+
+    def render(self, bytesio: io.BytesIO) -> None:
+        raise NotImplementedError()
+
+    # For MetadataConfig (for subclasses that extend it)
+    def global_config(self) -> "datacube_ows.ows_configuration.OWSConfig":
+        return self.style.global_config()
+
+    def get_obj_label(self) -> str:
+        """Return the metadata path prefix for this object."""
+        style_label: str = self.style.get_obj_label()
+        if self.style == self.style_or_mdh:
+            min_count: int = 1
+        else:
+            min_count = self.style_or_mdh.min_count
+        return f"{style_label}.legend.{min_count}"
+
+
 class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
     """"
     Base Class from which all style classes are extended.
@@ -41,8 +105,13 @@ class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
     INDEX_KEYS = ["layer", "style"]
 
     # For OWSMetaDataConfig
-    default_title = "Stand-Alone Style"
-    default_abstract = "Stand-Alone Style"
+    @property
+    def default_title(self) -> Optional[str]:
+        return "Stand-Alone Style"
+
+    @property
+    def default_abstract(self) -> Optional[str]:
+        return "Stand-Alone Style"
 
     # Over-ridden by subclasses that support auto-legends
     auto_legend: bool = False
@@ -128,7 +197,7 @@ class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
         self.declare_unready("needed_bands")
         self.declare_unready("flag_bands")
 
-        self.parse_legend_cfg(cast(CFG_DICT, raw_cfg.get("legend", {})))
+        self.legend_cfg = self.Legend(self, raw_cfg.get("legend", {}))
         if not defer_multi_date:
             self.parse_multi_date(raw_cfg)
 
@@ -314,53 +383,29 @@ class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
         """
         raise NotImplementedError()
 
-    # pylint: disable=attribute-defined-outside-init
-    def parse_legend_cfg(self, cfg: CFG_DICT) -> None:
-        """Used by __init__()"""
-        self.show_legend = cast(bool, cfg.get("show_legend", self.auto_legend))
-        self.legend_url_override = cast(Optional[str], cfg.get('url', None))
-        self.legend_cfg = cfg
-
     def render_legend(self, dates: Union[int, List[Any]]) -> Optional["PIL.Image.Image"]:
         """
         Render legend, if possible
         :param dates: The number of dates to render the legend for (e.g. for delta)
         :return: A PIL Image object, or None.
         """
-        mdh = self.get_multi_date_handler(dates)
-        url = self.legend_override_with_url(mdh)
-        if url:
+        legend = self.get_legend_cfg(dates)
+        if not legend.show_legend:
+            return None
+        if legend.legend_urls:
+            locale = get_locale().language
+            locales = self.global_config().locales
+            if locale not in self.global_config().locales:
+                locale = self.global_config().default_locale
+            url = legend.legend_urls[locale]
             return get_image_from_url(url)
-        if not self.auto_legend:
+        if not legend.style_or_mdh.auto_legend:
             return None
         bytesio = io.BytesIO()
-        if mdh:
-            mdh.legend(bytesio)
-        else:
-            self.single_date_legend(bytesio)
+        legend.render(bytesio)
         bytesio.seek(0)
         return Image.open(bytesio)
 
-    def single_date_legend(self, bytesio: io.BytesIO) -> None:
-        """
-        Write a legend into a bytes buffer as a PNG image.
-
-        Overridden by subclasses.
-
-        :param bytesio:  io.BytesIO byte buffer.
-        """
-        raise NotImplementedError()
-
-    def legend_override_with_url(self, mdh: Optional["StyleDefBase.MultiDateHandler"] = None) -> Optional[str]:
-        """
-        Find appropriate overide URL
-        :param mdh: Optional multidatehandler.  If None, use default single date override url.
-        :return: A URL string, or None
-        """
-        if mdh:
-            return mdh.legend_url_override
-        return self.legend_url_override
-
     @staticmethod
     def count_dates(count_or_sized_or_ds: Union[int, Sized, xr.Dataset]) -> int:
         if isinstance(count_or_sized_or_ds, int):
@@ -374,6 +419,13 @@ class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
         else:
             return len(count_or_sized_or_ds)
 
+    def get_legend_cfg(self, count_or_sized_or_ds: Union[int, Sized, xr.Dataset]
+                               ) -> LegendBase:
+        mdh = self.get_multi_date_handler(count_or_sized_or_ds)
+        if mdh:
+            return mdh.legend_cfg
+        return self.legend_cfg
+
     def get_multi_date_handler(self, count_or_sized_or_ds: Union[int, Sized, xr.Dataset]
                                ) -> Optional["StyleDefBase.MultiDateHandler"]:
         """
@@ -419,6 +471,14 @@ class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
                     return sub
         return None
 
+    # pylint: disable=abstract-method
+    class Legend(LegendBase):
+        """
+        Style Legend class.
+
+        May be overridden by style subclasses to support autolegend generation
+        """
+
     class MultiDateHandler(OWSConfigEntry):
         """
         MultiDateHandler base class.
@@ -462,7 +522,7 @@ class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
                 self.aggregator = None
                 if self.non_animate_requires_aggregator:
                     raise ConfigException("Aggregator function is required for non-animated multi-date handlers.")
-            self.parse_legend_cfg(cast(CFG_DICT, cfg.get("legend", {})))
+            self.legend_cfg = self.Legend(self, raw_cfg.get("legend", {}))
             self.preserve_user_date_order = cast(bool, cfg.get("preserve_user_date_order", False))
 
         def applies_to(self, count: int) -> bool:
@@ -488,17 +548,12 @@ class StyleDefBase(OWSExtensibleConfigEntry, OWSMetadataConfig):
             """
             return self.style.transform_single_date_data(data)
 
-        # pylint: disable=attribute-defined-outside-init
-        def parse_legend_cfg(self, cfg: CFG_DICT) -> None:
-            self.show_legend = cast(bool, cfg.get("show_legend", self.auto_legend))
-            self.legend_url_override = cast(Optional[str], cfg.get('url', None))
-            self.legend_cfg = cfg
-
-        def legend(self, bytesio: io.BytesIO) -> None:
+        # pylint: disable=abstract-method
+        class Legend(LegendBase):
             """
-            Write a legend as a png to a bytesio buffer.
-            
-            :param bytesio: 
+            MultiDateHandler Legend class.
+
+            May be overridden by MDH subclasses to support autolegend generation
             """
 
 
@@ -573,6 +628,8 @@ class BandIdxProxy:
 
 class GlobalCfgProxy:
     internationalised = False
+    default_locale = "en"
+    locales = ["en"]
 
 
 class StandaloneProductProxy:
