@@ -24,7 +24,7 @@ except ImportError:
 
 from xarray import Dataset
 
-from datacube_ows.config_utils import CFG_DICT
+from datacube_ows.config_utils import CFG_DICT, OWSMetadataConfig
 from datacube_ows.ogc_utils import ConfigException, FunctionWrapper
 from datacube_ows.styles.base import StyleDefBase
 from datacube_ows.styles.expression import Expression
@@ -164,90 +164,13 @@ def read_mpl_ramp(mpl_ramp: str) -> RAMP_SPEC:
     return unscaled_cmap
 
 
-def colour_ramp_legend(bytesio: io.BytesIO,
-                       legend_cfg: CFG_DICT,
-                       colour_ramp: "ColorRamp",
-                       map_name: str,
-                       default_title: str) -> None:
-    """
-    Generate a matplotlib legend for a colour ramp
-
-    :param bytesio: A BytesIO object to write the legend image into
-    :param legend_cfg: Legend configuration
-    :param colour_ramp: A ColorRamp object
-    :param map_name: a name for the transient MPL colourmap object
-    :param default_title: The default title to use (used if no title is set in legend_cfg)
-    """
-    def create_cdict_ticks(cfg: CFG_DICT, ramp: "ColorRamp") -> Tuple[
-        MutableMapping[str, List[Tuple[float, float, float]]],
-        MutableMapping[float, str],
-    ]:
-        normalize_factor = float(ramp.legend_end) - float(ramp.legend_begin)
-
-        cdict = cast(MutableMapping[str, List[Tuple[float, float, float]]], dict())
-        bands = cast(MutableMapping[str, List[Tuple[float, float, float]]], defaultdict(list))
-        started = False
-        finished = False
-        for index, ramp_point in enumerate(ramp.ramp):
-            if finished:
-                continue
-
-            value = cast(Union[float, int], ramp_point.get("value"))
-            normalized = (value - float(ramp.legend_begin)) / float(normalize_factor)
-
-            if not started:
-                if isclose(value, float(ramp.legend_begin), abs_tol=1e-9):
-                    started = True
-                else:
-                    continue
-            if not finished:
-                if isclose(value, float(ramp.legend_end), abs_tol=1e-9):
-                    finished = True
-
-            for band, intensity in ramp.components.items():
-                bands[band].append((normalized, intensity[index], intensity[index]))
-
-        for band, blist in bands.items():
-            cdict[band] = blist
-
-        ticks = cast(MutableMapping[float, str], dict())
-        for tick, tick_lbl in zip(ramp.ticks, ramp.tick_labels):
-            value = float(tick)
-            normalized = (value - float(ramp.legend_begin)) / float(normalize_factor)
-            ticks[normalized] = tick_lbl   # REVISIT: map on float???
-
-        return cdict, ticks
-
-    cdict, ticks = create_cdict_ticks(legend_cfg, colour_ramp)
-
-    plt.rcdefaults()
-    if colour_ramp.legend_mpl_rcparams:
-        plt.rcParams.update(colour_ramp.legend_mpl_rcparams)
-    fig = plt.figure(figsize=(colour_ramp.legend_width, colour_ramp.legend_height))
-    ax = fig.add_axes(colour_ramp.legend_strip_location)
-    custom_map = LinearSegmentedColormap(map_name, cdict)
-    color_bar = matplotlib.colorbar.ColorbarBase(
-        ax,
-        cmap=custom_map,
-        orientation="horizontal")
-
-    color_bar.set_ticks(list(ticks.keys()))
-    color_bar.set_ticklabels(list(ticks.values()))
-
-    title = colour_ramp.legend_title if colour_ramp.legend_title else default_title
-    if colour_ramp.legend_units:
-        title = title + "(" + colour_ramp.legend_units + ")"
-
-    color_bar.set_label(title)
-
-    plt.savefig(bytesio, format='png')
-
-
 class ColorRamp:
     """
     Represents a colour ramp for image and legend rendering purposes
     """
-    def __init__(self, style: StyleDefBase, ramp_cfg: CFG_DICT) -> None:
+    def __init__(self, style: StyleDefBase,
+                       ramp_cfg: CFG_DICT,
+                       legend: "RampLegendBase") -> None:
         """
         :param style: The style owning the ramp
         :param ramp_cfg: Style config
@@ -262,74 +185,59 @@ class ColorRamp:
                 unscaled_ramp = read_mpl_ramp(cast(str, ramp_cfg["mpl_ramp"]))
             raw_scaled_ramp = scale_unscaled_ramp(rmin, rmax, unscaled_ramp)
         self.ramp = raw_scaled_ramp
-        legend_cfg = cast(CFG_DICT, ramp_cfg.get("legend", {}))
-
-        # Legend typing
-        self.legend_begin = Decimal("nan")
-        self.legend_end = Decimal("nan")
-        self.ticks = cast(List[Decimal], [])
-        self.tick_labels = cast(List[str], [])
-        self.legend_mpl_rcparams = cast(MutableMapping[str, str], {})
-        self.legend_width = 0.0
-        self.legend_height = 0.0
-        self.legend_strip_location = cast(List[float], [])
-        self.legend_title = ""
-        self.legend_units = ""
-        self.legend_decimal_places = Decimal("nan")
-        self.rounder = Decimal("nan")
-        if legend_cfg.get("show_legend", True) and not legend_cfg.get("url"):
-            self.parse_legend(legend_cfg)
-        else:
-            self.auto_legend = False
 
         self.values = cast(List[float], [])
         self.components = cast(MutableMapping[str, List[float]], {})
         self.crack_ramp()
 
-        if self.auto_legend:
-            fbegin = float(self.legend_begin)
-            fend = float(self.legend_end)
-            begin_in_ramp = False
-            end_in_ramp = False
-            begin_before_idx = None
-            end_before_idx = None
+        # Handle the mutual interdepencies between the ramp and the legend
+        # 1. Let legend read its defaults from this ramp if needed
+        legend.register_ramp(self)
+        # 2. Extend our colour ramp to support legend if needed
+        if self.style.auto_legend:
+            fleg_begin = float(legend.begin)
+            fleg_end = float(legend.end)
+            leg_begin_in_ramp = False
+            leg_end_in_ramp = False
+            leg_begin_before_idx = None
+            leg_end_before_idx = None
             for idx, col_point in enumerate(self.ramp):
                 col_val = col_point["value"]
-                if not begin_in_ramp and begin_before_idx is None:
-                    if isclose(col_val, fbegin, abs_tol=1e-9):
-                        begin_in_ramp = True
-                    elif col_val > fbegin:
-                        begin_before_idx = idx
-                if not end_in_ramp and end_before_idx is None:
-                    if isclose(col_val, fend, abs_tol=1e-9):
+                if not leg_begin_in_ramp and leg_begin_before_idx is None:
+                    if isclose(col_val, fleg_begin, abs_tol=1e-9):
+                        leg_begin_in_ramp = True
+                    elif col_val > fleg_begin:
+                        leg_begin_before_idx = idx
+                if not leg_end_in_ramp and leg_end_before_idx is None:
+                    if isclose(col_val, fleg_end, abs_tol=1e-9):
                         end_in_ramp = True
-                    elif col_val > fend:
+                    elif col_val > fleg_end:
                         end_before_idx = idx
-            if not begin_in_ramp:
-                color, alpha = self.color_alpha_at(fbegin)
+            if not leg_begin_in_ramp:
+                color, alpha = self.color_alpha_at(fleg_begin)
                 begin_col_point = {
-                    "value": fbegin,
+                    "value": fleg_begin,
                     "color": color.get_hex(),
                     "alpha": alpha
                 }
-                if begin_before_idx is None:
+                if leg_begin_before_idx is None:
                     self.ramp.append(begin_col_point)
                 else:
-                    self.ramp.insert(begin_before_idx, begin_col_point)
-                if end_before_idx is not None:
-                    end_before_idx += 1
-            if not end_in_ramp:
-                color, alpha = self.color_alpha_at(fend)
+                    self.ramp.insert(leg_begin_before_idx, begin_col_point)
+                if leg_end_before_idx is not None:
+                    leg_end_before_idx += 1
+            if not leg_end_in_ramp:
+                color, alpha = self.color_alpha_at(fleg_end)
                 end_col_point = {
-                    "value": fend,
+                    "value": fleg_end,
                     "color": color.get_hex(),
                     "alpha": alpha
                 }
-                if end_before_idx is None:
+                if leg_end_before_idx is None:
                     self.ramp.append(end_col_point)
                 else:
-                    self.ramp.insert(end_before_idx, end_col_point)
-            if not end_in_ramp or not begin_in_ramp:
+                    self.ramp.insert(leg_end_before_idx, end_col_point)
+            if not leg_end_in_ramp or not leg_begin_in_ramp:
                 self.crack_ramp()
 
     def crack_ramp(self) -> None:
@@ -342,127 +250,6 @@ class ColorRamp:
             "alpha": a
         }
 
-    def parse_legend(self, cfg: CFG_DICT) -> None:
-        def rounder_str(prec: int) -> str:
-            rstr = "1"
-            if prec == 0:
-                return rstr
-            rstr += "."
-            for i in range(prec - 1):
-                rstr += "0"
-            rstr += "1"
-            return rstr
-
-        self.auto_legend = True
-        self.legend_title = cfg.get("title")
-        self.legend_units = cfg.get("units", "")
-        self.legend_decimal_places = cfg.get("decimal_places", 1)
-        if self.legend_decimal_places < 0:
-            raise ConfigException("decimal_places cannot be negative")
-        self.rounder = Decimal(rounder_str(self.legend_decimal_places))
-        self.parse_legend_range(cfg)
-        self.parse_legend_ticks(cfg)
-        self.parse_legend_tick_labels(cfg)
-        self.parse_legend_matplotlib_args(cfg)
-        # Check for old style config - was deprecated, now not supported
-        legend_legacy = any(
-            legent in cfg
-            for legent in ["major_ticks", "offset", "scale_by", "radix_point"]
-        )
-        if not legend_legacy and all(
-            legent not in cfg
-            for legent in ["begin", "end", "decimal_places", "ticks_every", "tick_count", "tick_labels", "ticks"]
-        ):
-            # No legacy entries, but no new entries either.
-            # Check ramp for legend tips
-            for r in self.ramp:
-                if "legend" in r:
-                    legend_legacy = True
-                    break
-        if legend_legacy:
-            raise ConfigException(
-                        "Style %s uses a no-longer supported format for legend configuration.  " +
-                        "Please refer to the documentation and update your config" % self.style.name)
-
-    def parse_legend_range(self, cfg: CFG_DICT) -> None:
-        if "begin" in cfg:
-            self.legend_begin = Decimal(cast(Union[str, float, int], cfg["begin"]))
-        else:
-            for col_def in self.ramp:
-                if isclose(col_def.get("alpha", 1.0), 1.0, abs_tol=1e-9):
-                    self.legend_begin = Decimal(col_def["value"])
-                    break
-            if self.legend_begin.is_nan():
-                self.legend_begin = Decimal(self.ramp[0]["value"])
-        if "end" in cfg:
-            self.legend_end = Decimal(cast(Union[str, float, int], cfg["end"]))
-        else:
-            for col_def in reversed(self.ramp):
-                if col_def.get("alpha", 1.0) == 1.0:
-                    self.legend_end = Decimal(col_def["value"])
-                    break
-            if self.legend_end.is_nan():
-                self.legend_end = Decimal(self.ramp[-1]["value"])
-
-    def parse_legend_ticks(self, cfg: CFG_DICT) -> None:
-        # Ticks
-        ticks_handled = False
-        if "ticks_every" in cfg:
-            if "tick_count" in cfg:
-                raise ConfigException("Cannot use tick count and ticks_every in the same legend")
-            if "ticks" in cfg:
-                raise ConfigException("Cannot use ticks and ticks_every in the same legend")
-            delta = Decimal(cast(Union[int, float, str], cfg["ticks_every"]))
-            tickval = self.legend_begin
-            while tickval < self.legend_end:
-                self.ticks.append(tickval)
-                tickval += delta
-            self.ticks.append(tickval)
-            ticks_handled = True
-        if "ticks" in cfg:
-            if "tick_count" in cfg:
-                raise ConfigException("Cannot use tick count and ticks in the same legend")
-            self.ticks = [Decimal(t) for t in cast(List[Union[str, int, float]], cfg["ticks"])]
-            ticks_handled = True
-        if not ticks_handled:
-            count = int(cast(Union[str, int], cfg.get("tick_count", 1)))
-            if count < 0:
-                raise ConfigException("tick_count cannot be negative")
-            elif count == 0:
-                self.ticks.append(self.legend_begin)
-            else:
-                delta = self.legend_end - self.legend_begin
-                dcount = Decimal(count)
-
-                for i in range(0, count + 1):
-                    tickval = self.legend_begin + (Decimal(i) / dcount) * delta
-                    self.ticks.append(tickval.quantize(self.rounder, rounding=ROUND_HALF_UP))
-
-    def parse_legend_tick_labels(self, cfg: CFG_DICT) -> None:
-        labels = cast(MutableMapping[str, MutableMapping[str, str]], cfg.get("tick_labels", {}))
-        defaults = labels.get("default", {})
-        default_prefix = defaults.get("prefix", "")
-        default_suffix = defaults.get("suffix", "")
-        # pylint: disable=attribute-defined-outside-init
-        for tick in self.ticks:
-            label_cfg = labels.get(str(tick))
-            if label_cfg:
-                prefix = label_cfg.get("prefix", default_prefix)
-                suffix = label_cfg.get("suffix", default_suffix)
-                label  = label_cfg.get("label", str(tick))
-                self.tick_labels.append(prefix + label + suffix)
-            else:
-                self.tick_labels.append(
-                    default_prefix + str(tick) + default_suffix
-                )
-
-    def parse_legend_matplotlib_args(self, cfg: CFG_DICT) -> None:
-        self.legend_mpl_rcparams = cast(MutableMapping[str, str], cfg.get("rcParams", {}))
-        self.legend_width = cast(float, cfg.get("width", 4.0))
-        self.legend_height = cast(float, cfg.get("height", 1.25))
-        self.legend_strip_location = cast(List[float],
-                                          cfg.get("strip_location", [0.05, 0.5, 0.9, 0.15]))
-
     def get_value(self, data: Union[float, "xarray.DataArray"], band: str) -> NDArray:
         return numpy.interp(data, self.values, self.components[band])
 
@@ -490,6 +277,223 @@ class ColorRamp:
         return color, alpha
 
 
+class RampLegendBase(StyleDefBase.Legend, OWSMetadataConfig):
+    METADATA_ABSTRACT: bool = False
+    METADATA_LEGEND_UNITS: bool = True
+    METADATA_TICK_LABELS: bool = True
+
+    def __init__(self, style_or_mdh: Union["StyleDefBase", "StyleDefBase.Legend"], cfg: CFG_DICT) -> None:
+        super().__init__(style_or_mdh, cfg)
+        raw_cfg = cast(CFG_DICT, self._raw_cfg)
+        # Range - defaults deferred until we have parsed the associated ramp
+        if "begin" not in raw_cfg:
+            self.begin = Decimal("nan")
+        else:
+            self.begin = Decimal(cast(Union[str, float, int], raw_cfg["begin"]))
+        if "end" not in raw_cfg:
+            self.end = Decimal("nan")
+        else:
+            self.end = Decimal(cast(Union[str, float, int], raw_cfg["end"]))
+
+        # decimal_places, rounder
+        def rounder_str(prec: int) -> str:
+            rstr = "1"
+            if prec == 0:
+                return rstr
+            rstr += "."
+            for i in range(prec - 1):
+                rstr += "0"
+            rstr += "1"
+            return rstr
+
+        self.decimal_places = raw_cfg.get("decimal_places", 1)
+        if self.decimal_places < 0:
+            raise ConfigException("decimal_places cannot be negative")
+        self.rounder = Decimal(rounder_str(self.decimal_places))
+
+        # Ticks - Non-explicit tick values deferred until we have parsed the associated ramp
+        ticks_handled = False
+        self.ticks_every: Optional[Decimal] = None
+        self.tick_count: Optional[int] = None
+        self.ticks: List[Decimal] = []
+        if "ticks_every" in raw_cfg:
+            if "tick_count" in raw_cfg:
+                raise ConfigException("Cannot use tick count and ticks_every in the same legend")
+            if "ticks" in raw_cfg:
+                raise ConfigException("Cannot use ticks and ticks_every in the same legend")
+            self.ticks_every = Decimal(cast(Union[int, float, str], raw_cfg["ticks_every"]))
+            if self.ticks_every.is_zero() or self.ticks_every.is_signed():
+                raise ConfigException("ticks_every must be greater than zero")
+            ticks_handled = True
+        if "ticks" in raw_cfg:
+            if "tick_count" in raw_cfg:
+                raise ConfigException("Cannot use tick count and ticks in the same legend")
+            self.ticks = [Decimal(t) for t in cast(List[Union[str, int, float]], raw_cfg["ticks"])]
+            ticks_handled = True
+        if not ticks_handled:
+            self.tick_count = int(cast(Union[str, int], raw_cfg.get("tick_count", 1)))
+            if self.tick_count < 0:
+                raise ConfigException("tick_count cannot be negative")
+        # prepare for tick labels
+        self.cfg_labels = cast(MutableMapping[str, MutableMapping[str, str]], raw_cfg.get("tick_labels", {}))
+        defaults = self.cfg_labels.get("default", {})
+        self.lbl_default_prefix = defaults.get("prefix", "")
+        self.lbl_default_suffix = defaults.get("suffix", "")
+        self.tick_labels: List[str] = []
+        # handle matplotlib args
+        self.strip_location = cast(List[float],
+                                   raw_cfg.get("strip_location", [0.05, 0.5, 0.9, 0.15]))
+        # throw error on legacy syntax
+        self.fail_legacy()
+
+    def fail_legacy(self) -> None:
+        if any(
+                legent in self._raw_cfg
+                for legent in ["major_ticks", "offset", "scale_by", "radix_point"]
+        ):
+            raise ConfigException(
+                f"Style {self.style.name} uses a no-longer supported format for legend configuration.  " +
+                "Please refer to the documentation and update your config")
+
+    def register_ramp(self, ramp: ColorRamp) -> None:
+        if self.begin.is_nan():
+            for col_def in ramp.ramp:
+                if isclose(col_def.get("alpha", 1.0), 1.0, abs_tol=1e-9):
+                    self.begin = Decimal(col_def["value"])
+                    break
+            if self.begin.is_nan():
+                self.begin = Decimal(ramp.ramp[0]["value"])
+        if self.end.is_nan():
+            for col_def in reversed(ramp.ramp):
+                if isclose(col_def.get("alpha", 1.0), 1.0, abs_tol=1e-9):
+                    self.end = Decimal(col_def["value"])
+                    break
+            if self.end.is_nan():
+                self.end = Decimal(ramp.ramp[-1]["value"])
+        for t in self.ticks:
+            if t < self.begin or t > self.end:
+                raise ConfigException("Explicit ticks must all be within legend begin/end range")
+        if self.ticks_every is not None:
+            tickval = self.begin
+            while tickval < self.end:
+                self.ticks.append(tickval)
+                tickval += self.ticks_every
+            self.ticks.append(self.end)
+        elif self.tick_count is not None:
+            if self.tick_count == 0:
+                self.ticks.append(self.begin)
+            else:
+                delta = self.end - self.begin
+                dcount = Decimal(self.tick_count)
+                for i in range(0, self.tick_count + 1):
+                    tickval = self.begin + (Decimal(i) / dcount) * delta
+                    self.ticks.append(tickval.quantize(self.rounder, rounding=ROUND_HALF_UP))
+
+        # handle tick labels
+        for tick in self.ticks:
+            label_cfg = self.cfg_labels.get(str(tick))
+            if label_cfg:
+                prefix = label_cfg.get("prefix", self.lbl_default_prefix)
+                suffix = label_cfg.get("suffix", self.lbl_default_suffix)
+                label = label_cfg.get("label", str(tick))
+                self.tick_labels.append(prefix + label + suffix)
+            else:
+                self.tick_labels.append(
+                    self.lbl_default_prefix + str(tick) + self.lbl_default_suffix
+                )
+        self.parse_metadata(self._raw_cfg)
+
+        # Check for legacy legend tips in ramp:
+        for r in ramp.ramp:
+            if "legend" in r:
+                raise ConfigException(
+                    f"Style {self.style.name} uses a no-longer supported format for legend configuration.  " +
+                    "Please refer to the documentation and update your config")
+
+    def tick_label(self, tick):
+        try:
+            tick_idx = self.ticks.index(tick)
+            metaval = self.read_local_metadata(f"lbl_{tick}")
+            if metaval:
+                return metaval
+            else:
+                return self.tick_labels[tick_idx]
+        except ValueError:
+            _LOG.error("'%s' is a not a valid tick", tick)
+            return None
+
+    def create_cdict_ticks(self) -> Tuple[
+        MutableMapping[str, List[Tuple[float, float, float]]],
+        MutableMapping[float, str],
+    ]:
+        normalize_factor = float(self.end) - float(self.begin)
+        cdict = cast(MutableMapping[str, List[Tuple[float, float, float]]], dict())
+        bands = cast(MutableMapping[str, List[Tuple[float, float, float]]], defaultdict(list))
+        started = False
+        finished = False
+        for index, ramp_point in enumerate(self.style_or_mdh.color_ramp.ramp):
+            if finished:
+                break
+
+            value = cast(Union[float, int], ramp_point.get("value"))
+            normalized = (value - float(self.begin)) / float(normalize_factor)
+
+            if not started:
+                if isclose(value, float(self.begin), abs_tol=1e-9):
+                    started = True
+                else:
+                    continue
+            if not finished:
+                if isclose(value, float(self.end), abs_tol=1e-9):
+                    finished = True
+
+            for band, intensity in self.style_or_mdh.color_ramp.components.items():
+                bands[band].append((normalized, intensity[index], intensity[index]))
+
+        for band, blist in bands.items():
+            cdict[band] = blist
+
+        ticks = cast(MutableMapping[float, str], dict())
+        for tick in self.ticks:
+            value = float(tick)
+            normalized = (value - float(self.begin)) / float(normalize_factor)
+            ticks[normalized] = self.tick_label(tick)
+
+        return cdict, ticks
+
+    def display_title(self):
+        if self.units:
+            return f"{self.title}({self.units})"
+        else:
+            return self.title
+
+    def plot_name(self):
+        return f"{self.style.product.name}_{self.style.name}"
+
+    def render(self, bytesio: io.BytesIO) -> None:
+        cdict, ticks = self.create_cdict_ticks()
+        plt.rcdefaults()
+        if self.mpl_rcparams:
+            plt.rcParams.update(self.mpl_rcparams)
+        fig = plt.figure(figsize=(self.width, self.height))
+        ax = fig.add_axes(self.strip_location)
+        custom_map = LinearSegmentedColormap(self.plot_name(), cdict)
+        color_bar = matplotlib.colorbar.ColorbarBase(
+            ax,
+            cmap=custom_map,
+            orientation="horizontal")
+        color_bar.set_ticks(list(ticks.keys()))
+        color_bar.set_ticklabels(list(ticks.values()))
+        color_bar.set_label(self.display_title())
+        plt.savefig(bytesio, format='png')
+
+    # For MetadataConfig
+    @property
+    def default_title(self) -> Optional[str]:
+        return self.style.title
+
+
+
 class ColorRampDef(StyleDefBase):
     """
     Colour ramp Style subclass
@@ -508,7 +512,7 @@ class ColorRampDef(StyleDefBase):
         super(ColorRampDef, self).__init__(product, style_cfg,
                            stand_alone=stand_alone, defer_multi_date=True, user_defined=user_defined)
         style_cfg = cast(CFG_DICT, self._raw_cfg)
-        self.color_ramp = ColorRamp(self, style_cfg)
+        self.color_ramp = ColorRamp(self, style_cfg, self.legend_cfg)
         self.include_in_feature_info = bool(style_cfg.get("include_in_feature_info", True))
 
         if "index_function" in style_cfg:
@@ -554,18 +558,8 @@ class ColorRampDef(StyleDefBase):
         d = self.apply_index(data)
         return self.color_ramp.apply(d)
 
-    def single_date_legend(self, bytesio: io.BytesIO) -> None:
-        """
-        Write a legend into a bytes buffer as a PNG image.
-
-        :param bytesio:  io.BytesIO byte buffer.
-        """
-        colour_ramp_legend(bytesio,
-                           self.legend_cfg,
-                           self.color_ramp,
-                           self.product.name,
-                           self.title      # pyre-ignore[16]
-                           )
+    class Legend(RampLegendBase):
+        pass
 
     class MultiDateHandler(StyleDefBase.MultiDateHandler):
         auto_legend = True
@@ -583,7 +577,7 @@ class ColorRampDef(StyleDefBase):
                 self.color_ramp = style.color_ramp
             else:
                 self.feature_info_label = cast(Optional[str], cfg.get("feature_info_label", None))
-                self.color_ramp = ColorRamp(style, cfg)
+                self.color_ramp = ColorRamp(style, cfg, self.legend_cfg)
 
         def transform_data(self, data: "xarray.Dataset") -> "xarray.Dataset":
             """
@@ -596,23 +590,9 @@ class ColorRampDef(StyleDefBase):
             agg = self.aggregator(xformed_data)
             return self.color_ramp.apply(agg)
 
-        def legend(self, bytesio: io.BytesIO) -> None:
-            """
-            Write a legend as a png to a bytesio buffer.
-
-            :param bytesio:
-            """
-            if self.animate and not self.legend_cfg:
-                self.style.single_date_legend(bytesio)
-            else:
-                title = self.legend_cfg.get("title", self.range_str() + " Dates")
-                name = self.style.product.name + f"_{self.min_count}"
-                colour_ramp_legend(bytesio,
-                                   self.legend_cfg,
-                                   self.color_ramp,
-                                   name,
-                                   title
-                                   )
+        class Legend(RampLegendBase):
+            def plot_name(self):
+                return f"{self.style.product.name}_{self.style.name}_{self.style_or_mdh.min_count}"
 
 # Register ColorRampDef as Style subclass.
 StyleDefBase.register_subclass(ColorRampDef, ("range", "color_ramp"))
